@@ -3,17 +3,17 @@ import hashlib
 import hmac
 import ipaddress
 import json
-import os
 from typing import List, Optional, Union
 
 from cryptography.fernet import Fernet
 from starlette.requests import Request
 
 from appodus_utils.common.commons import Utils
+from appodus_utils.common.utils_settings import utils_settings
 from appodus_utils.exception.exceptions import ForbiddenException, UnauthorizedException
 
-client_secret_encryption_key = os.getenv('APPODUS_CLIENT_SECRET_ENCRYPTION_KEY')
-client_request_expires_seconds: int = int(os.getenv('APPODUS_CLIENT_REQUEST_EXPIRES_SECONDS', 300))
+client_secret_encryption_key = utils_settings.APPODUS_CLIENT_SECRET_ENCRYPTION_KEY
+client_request_expires_seconds = utils_settings.APPODUS_CLIENT_REQUEST_EXPIRES_SECONDS or 300
 fernet = Fernet(client_secret_encryption_key)
 
 class ClientUtils:
