@@ -1,4 +1,3 @@
-import enum
 import base64
 import enum
 import hashlib
@@ -9,7 +8,7 @@ import random
 import re
 import uuid
 from _decimal import Decimal
-from datetime import datetime, timedelta, timezone
+from datetime import datetime, timedelta, timezone, time
 from typing import List, Union, Any, Dict, Optional, Literal
 
 import inflection
@@ -29,11 +28,71 @@ MAX_ATTACHMENT_SIZE = 10 * 1024 * 1024
 CHUNK_SIZE = 64 * 1024  # 64 KB for streaming
 
 
+class WeekNumber(enum.IntEnum):
+    """Enum representing the weeks in a month for scheduling."""
+    WEEK_1 = 1
+    WEEK_2 = 2
+    WEEK_3 = 3
+    WEEK_4 = 4
+
 class Utils:
     default_max_miles_per_day = 200 * 365
     default_max_allowed_miles = 200 * 365
     pwd_context = CryptContext(schemes=["sha256_crypt"])
 
+    @staticmethod
+    def normalize_phone(number: str | int) -> str:
+        number_str = str(number)
+        digits = re.sub(r'[^\d]', '', number_str)
+        return f'+{digits}'
+
+    @staticmethod
+    def get_from_env(env_key: str, default: Optional[any] = None):
+        value = os.getenv(env_key)
+
+        return value or default
+
+    @staticmethod
+    def get_from_env_fail_if_not_exists(env_key: str, default: Optional[any] = None):
+        env_value = Utils.get_from_env(env_key=env_key, default=default)
+        if env_value:
+            return env_value
+
+        raise ValueError(f"Env value with key '{env_key}' not set.")
+
+    @staticmethod
+    def replace_case_insensitive(text: str, old: str, new: str):
+        pattern = re.compile(old, re.IGNORECASE)
+        new_text = pattern.sub(new, text)
+
+        return new_text
+
+    @staticmethod
+    def get_monthly_week_number(week_index: int) -> WeekNumber:
+        """
+        Given a global week index (1 to 52+), return the WeekNumber enum
+        corresponding to a 4-week repeating monthly cycle.
+
+        Example:
+            1  => WeekNumber.WEEK_1
+            2  => WeekNumber.WEEK_2
+            3  => WeekNumber.WEEK_3
+            4  => WeekNumber.WEEK_4
+            5  => WeekNumber.WEEK_1
+            6  => WeekNumber.WEEK_2
+            9  => WeekNumber.WEEK_1
+            17 => WeekNumber.WEEK_1
+            20 => WeekNumber.WEEK_4
+
+        Args:
+            week_index (int): The global week index (starting from 1).
+
+        Returns:
+            WeekNumber: The week number in the 4-week cycle.
+        """
+        week_num = ((week_index - 1) % 4) + 1
+        return WeekNumber(week_num)
+
     # Datetime
     @staticmethod
     def datetime_from_epoch(epoch_timestamp: Union[int, float], tz: timezone =timezone.utc) -> datetime:
@@ -134,6 +193,53 @@ class Utils:
         current_minus = Utils.datetime_now_minus(seconds=minus_seconds).timestamp()
         return current_minus < target_ts
 
+    @staticmethod
+    def obj_time_to_str(in_obj: Union[dict, Any]):
+        supplementary_dict = {}
+
+        if not in_obj or isinstance(in_obj, str):
+            return in_obj
+
+        if not isinstance(in_obj, dict):
+            in_obj = in_obj.model_dump()
+
+        in_obj_copy = in_obj.copy()
+        for field in in_obj:
+            try:
+
+                if isinstance(in_obj[field], dict):
+                    converted_dic = Utils.obj_time_to_str(in_obj[field])
+                    supplementary_dict.setdefault(field, converted_dic)
+                elif isinstance(in_obj[field], list):
+                    if len(in_obj[field]) < 1: # empty list
+                        supplementary_dict.setdefault(field, [])
+                    for obj in in_obj[field]:
+                        if isinstance(obj, time):
+                            converted_list = obj.isoformat()
+                        elif isinstance(obj, list) or isinstance(obj, dict):
+                            converted_list = Utils.obj_time_to_str(obj)
+                        else:
+                            converted_list = obj
+
+                        existing_values: list = supplementary_dict.get(field)
+                        if existing_values:
+                            existing_values.append(converted_list)
+                        else:
+                            existing_values = [converted_list]
+                        supplementary_dict.setdefault(field, existing_values)
+                elif isinstance(in_obj[field], time):
+                    converted = in_obj[field].isoformat()
+                    supplementary_dict.setdefault(field, converted)
+                else:
+                    supplementary_dict.setdefault(field, in_obj[field])
+                in_obj_copy.pop(field)
+            except AttributeError as e:
+                # TODO logger.error
+                raise e
+
+        supplementary_dict.update(in_obj_copy)
+        return supplementary_dict
+
     @staticmethod
     def template_bind_context(content: str, context: Dict[str, Any]) -> str:
         template = Template(content)
